

/**************************************************************
*
* draw.c source: methods drawing
*
**************************************************************/


#include "draw.h"

inline void drawPixel(u16 *video_buffer, u16 x, u16 y, u16 color) {

	u32 index = (y * SCREEN_WIDTH) + x;
	video_buffer[index] = color;
	 
}

inline u16 getPixel(u16 *video_buffer, u16 x, u16 y){
	u32 index = (y * SCREEN_WIDTH) + x;
	u16 color = video_buffer[index];
	return color;
}

//drawRotatedBitMap
void drawRotatedBitMap(u16* video_buffer_Source, u16* video_buffer_Dest, u16 bufferX, u16 bufferY, u16* bitmap, u8 bitmapX, u8 bitmapY, u16 width, u8 height, float angle){
	u16 index;
	//float angle = .1717;
	float cosine = cos(angle);
	float sine = sin(angle);

	// Compute dimensions of the resulting bitmap

	int cx = width/2;
	int cy = height/2;
	
	int x0 = (int)(0 - cx);
	int y0 = (int)(0 - cy);
	int x1 = (int)(width - cx);
	int y1 = (int)(0 - cy);
	int x2 = (int)(width - cx);
	int y2 = (int)(height - cy);
	int x3 = (int)(0 - cx);
	int y3 = (int)(height - cy);
	int x0N = (int)(x0 * cosine - y0 * sine) + cx;
	int y0N = (int)(y0 * cosine + x0 * sine) + cy;
	int x1N = (int)(x1 * cosine - y1 * sine) + cx;
	int y1N = (int)(y1 * cosine + x1 * sine) + cy;
	int x2N = (int)(x2 * cosine - y2 * sine) + cx;
	int y2N = (int)(y2 * cosine + x2 * sine) + cy;
	int x3N = (int)(x3 * cosine - y3 * sine) + cx;
	int y3N = (int)(y3 * cosine + x3 * sine) + cy;

	int minx = min(x0N,min(x1N, min(x2N,x3N))) ;
	int miny = min(y0N,min(y1N, min(y2N,y3N))) ;
	int maxx = max(x0N,max(x1N, max(x2N,x3N))) ;
	int maxy = max(y0N,max(y1N, max(y2N,y3N))) ;

	int x,y;
	for(  y = miny - 1  ; y <=  maxy + 1  ; y++ )
	{
		for(  x = minx - 1  ; x <=  maxx + 1  ; x++ )
		{
			int sourcex = (int)(( x - cx )*cosine + ( y - cy )*sine) + cx;
			int sourcey = (int)(( y - cy )*cosine - ( x - cx )*sine) + cy;
			
			int intBool1 = pointInTriangle ( x0N,x1N,x2N,y0N,y1N,y2N,x, y) ;
			int intBool2 = pointInTriangle ( x2N,x3N,x0N,y2N,y3N,y0N,x, y) ;
			
			if ( intBool1 == 1 || intBool2 == 1 )
			{
			if ( bufferX + x>0 && bufferX + x < 256  && bufferY + y>0 && bufferY + y < 192)
				{
				index = ((bitmapY + sourcey) * width) + bitmapX + sourcex;
				drawPixel(video_buffer_Dest, bufferX + x, bufferY + y, PIXEL(bitmap[index]));
				}
				}
				else
				{
				u16 backColor = getPixel(video_buffer_Source, bufferX + x, bufferY + y);
				drawPixel(video_buffer_Dest, bufferX + x, bufferY + y, backColor );
				}
		}
	}

}

int pointInTriangle( int x1, int x2, int x3, int y1, int y2, int y3, int x4, int y4 )
{
	int a,b,c;
    a = (x1 - x4)*(y2 - y4) - (x2 - x4)*(y1 - y4);
    b = (x2 - x4)*(y3 - y4) - (x3 - x4)*(y2 - y4);
    c = (x3 - x4)*(y1 - y4) - (x1 - x4)*(y3 - y4);
    return (sign(a) == sign(b) && sign(b) == sign(c));
       
       }
	   
int sign( int myNumber ){

	if ( myNumber >= 0) return 1;
	return -1;
}
	   

void drawBitmapWithBlanks(u16* video_buffer_Source, u16* video_buffer_Dest, u16 bufferX, u16 bufferY, u16* bitmap, u8 bitmapX, u8 bitmapY, u16 width, u8 height, float angle) {
	u16 i, j, index;
	
	//float angle = .1717;
	float cosine = (float)cos(angle);
	float sine = (float)sin(angle);
	
	for(i = 0; i < height; i++) {
		for(j = 0; j < width; j++) {
			index = ((bitmapY + i) * width) + bitmapX + j;
			//minus point of rotation
			float x = (float)j - height/2.0;
			float y = (float)i - width/2.0;
			//rotating matrix
			float newX = x*cosine-y*sine;
			float newY = x*sine+y*cosine;
			//plus point of rotation
			int newIntX = (int)newX + height/2.0;
			int newIntY = (int)newY + width/2.0;
			if ( bufferX + newIntX>0 && bufferX + newIntX < 256  && bufferY + newIntY>0 && bufferY + newIntY < 192)
				drawPixel(video_buffer_Dest, bufferX + newIntX, bufferY + newIntY, PIXEL(bitmap[index]));
		}
	}
}




void drawLine( u16* video_buffer, int x1, int y1, int x2, int y2)
{

	u16 color  = RGBColor(31*rand(),31*rand(),31*rand());
	
    int yStep = SCREEN_WIDTH;
    int xStep = 1;      
    int xDiff = x2 - x1;
    int yDiff = y2 - y1;
 
    int errorTerm = 0;
    int offset = y1 * SCREEN_WIDTH + x1; 
    int i; 
    
    //need to adjust if y1 > y2
    if (yDiff < 0)       
    {                  
       yDiff = -yDiff;   //absolute value
       yStep = -yStep;   //step up instead of down   
    }
    
    //same for x
    if (xDiff < 0) 
    {           
       xDiff = -xDiff;            
       xStep = -xStep;            
    }        
 
    //case for changes more in X than in Y	 
    if (xDiff > yDiff) 
    {                            
       for (i = 0; i < xDiff + 1; i++)
       {                           
          video_buffer[offset] = color;  
 
          offset += xStep;           
 
          errorTerm += yDiff;     
 
          if (errorTerm > xDiff) 
          {  
             errorTerm -= xDiff;     
             offset += yStep;        
          }
       }
    }//end if xdiff > ydiff
    //case for changes more in Y than in X
    else 
    {                       
       for (i = 0; i < yDiff + 1; i++) 
       {  
          video_buffer[offset] = color;  
 
          offset += yStep;           
 
          errorTerm += xDiff;    
 
          if (errorTerm > yDiff) 
          {     
             errorTerm -= yDiff;  
             offset += xStep;     
 
          }
       }
    }
 
}

float dist ( int i, int j,  int mx, int my ){

	return  sqrt((i-mx)*(i-mx) + (j-my)*(j-my));

}

extern void drawBackBall( u16* video_buffer_Source, u16* video_buffer_Dest, int x1, int y1, float myRadius){
	int i,j;
	for (i= x1 - myRadius; i<x1 + myRadius ; i++)
		for (j= y1 - myRadius; j<y1+ myRadius ; j++)
			if ( i>0 && i < 256 && j>0 && j < 192)
			{
				u16 backColor = getPixel(video_buffer_Source, i, j);
				drawPixel(video_buffer_Dest, i, j, backColor );
			}
}


extern void drawBall( u16* video_buffer, int x1, int y1, float myRadius , u16 color  ){

	int i, j;

	for (i= x1 - myRadius; i<x1 + myRadius ; i++)
		for (j= y1 - myRadius; j<y1+ myRadius ; j++)
		if ( i>0 && i < 256  && j>0 && j < 192)
			if ( dist ( i, j, x1, y1 ) < myRadius )
				drawPixel(video_buffer, i, j, color);

}


